"""
Tools for nonparametric statistics, mainly density estimation and regression.

For an overview of this module, see docs/source/nonparametric.rst
"""

from statsmodels import PytestTester
test = PytestTester()




from statsmodels.compat.testing import skipif

import numpy as np
import pandas as pd
from statsmodels.graphics.agreement import mean_diff_plot


try:
    import matplotlib.pyplot as plt
    have_matplotlib = True
except:
    have_matplotlib = False


@skipif(not have_matplotlib, reason='matplotlib not available')
def test_mean_diff_plot():

    # Seed the random number generator.
    # This ensures that the results below are reproducible.
    np.random.seed(11111)
    m1 = np.random.random(20)
    m2 = np.random.random(20)

    fig = plt.figure()
    ax = fig.add_subplot(111)

    # basic test.
    fig = mean_diff_plot(m1, m2, ax=ax)
    plt.close(fig)

    # Test with pandas Series.
    p1 = pd.Series(m1)
    p2 = pd.Series(m2)
    fig = mean_diff_plot(p1, p2)
    plt.close(fig)

    # Test plotting on assigned axis.
    fig, ax = plt.subplots(2)
    mean_diff_plot(m1, m2, ax = ax[0])
    plt.close(fig)

    # Test the setting of confidence intervals.
    fig = mean_diff_plot(m1, m2, sd_limit = 0)
    plt.close(fig)

    # Test asethetic controls.
    fig = mean_diff_plot(m1, m2, scatter_kwds={'color':'green', 's':10})
    plt.close(fig)

    fig = mean_diff_plot(m1, m2, mean_line_kwds={'color':'green', 'lw':5})
    plt.close(fig)

    fig = mean_diff_plot(m1, m2, limit_lines_kwds={'color':'green',
                                                'lw':5,
                                                'ls':'dotted'})
    plt.close(fig)



"""
Compatibility shims to facilitate the transition from nose to pytest.  Can be removed once pytest
is the only way to run tests.
"""
from statsmodels.compat.python import PY3
import functools

has_pytest = True
try:
    import pytest
except ImportError:
    has_pytest = False

if PY3:
    SKIP_TYPES = type
else:
    import types

    SKIP_TYPES = (type, types.ClassType)

try:
    from unittest.case import SkipTest
except ImportError:
    try:
        from nose import SkipTest
    except ImportError:
        try:
            from unittest2 import SkipTest
        except ImportError:
            raise ImportError('Unable to locate SkipTest.  unittest, unittest2 or nose required.')


def skip(reason):
    def decorator(test_item):
        if not hasattr(test_item, '__name__') and has_pytest:
            return pytest.mark.skip(reason)
        if not isinstance(test_item, SKIP_TYPES):
            @functools.wraps(test_item)
            def skip_wrapper(*args, **kwargs):
                raise SkipTest(reason)

            test_item = skip_wrapper

        test_item.__unittest_skip__ = True
        test_item.__unittest_skip_why__ = reason
        return test_item

    return decorator


def _id(obj):
    return obj


def skipIf(condition, reason):
    """
    Skip a test if the condition is true.
    """
    if condition:
        return skip(reason)
    return _id


skipif = skipIf


def example(t):
    """
    Label a test as an example.

    Parameters
    ----------
    t : callable
        The test to label as slow.

    Returns
    -------
    t : callable
        The decorated test `t`.

    Examples
    --------
    The `statsmodels.compat.testing` module includes ``example``.
    A test can be decorated as slow like this::

      from statsmodels.compat.testing import example

      @example
      def test_example(self):
          print('Running an example')
    """

    t.example = True
    return t


__all__ = ['skip', 'skipif', 'SkipTest', 'example']



from collections import namedtuple
import numpy as np
from statsmodels.tools.tools import Bunch

_MinimalWLSModel = namedtuple('_MinimalWLSModel', ['weights'])


class _MinimalWLS(object):
    """
    Minimal implementation of WLS optimized for performance.

    Parameters
    ----------
    endog : array-like
        1-d endogenous response variable. The dependent variable.
    exog : array-like
        A nobs x k array where `nobs` is the number of observations and `k`
        is the number of regressors. An intercept is not included by default
        and should be added by the user. See
        :func:`statsmodels.tools.add_constant`.
    weights : array-like, optional
        1d array of weights.  If you supply 1/W then the variables are pre-
        multiplied by 1/sqrt(W).  If no weights are supplied the default value
        is 1 and WLS reults are the same as OLS.

    Notes
    -----
    Need resid, scale, fittedvalues, model.weights!
        history['scale'].append(tmp_results.scale)
        if conv == 'dev':
            history['deviance'].append(self.deviance(tmp_results))
        elif conv == 'sresid':
            history['sresid'].append(tmp_results.resid/tmp_results.scale)
        elif conv == 'weights':
            history['weights'].append(tmp_results.model.weights)
    Does not perform and checks on the input data
    """

    def __init__(self, endog, exog, weights=1.0):
        self.endog = endog
        self.exog = exog
        self.weights = weights
        w_half = np.sqrt(weights)

        self.wendog = w_half * endog
        if np.isscalar(weights):
            self.wexog = w_half * exog
        else:
            self.wexog = w_half[:, None] * exog

    def fit(self, method='pinv'):
        """
        Minimal implementation of WLS optimized for performance.

        Parameters
        ----------
        method : str, optional
            Method to use to estimate parameters.  "pinv", "qr" or "lstsq"

              * "pinv" uses the Moore-Penrose pseudoinverse
                 to solve the least squares problem.
              * "qr" uses the QR factorization.
              * "lstsq" uses the least squares implementation in numpy.linalg

        Returns
        -------
        results : namedtuple
            Named tuple containing the fewest terms needed to implement
            iterative estimation in models. Currently

              * params : Estimated parameters
              * fittedvalues : Fit values using original data
              * resid : Residuals using original data
              * model : namedtuple with one field, weights
              * scale : scale computed using weighted residuals

        Notes
        -----
        Does not perform and checks on the input data

        See Also
        --------
        statsmodels.regression.linear_model.WLS
        """
        if method == 'pinv':
            pinv_wexog = np.linalg.pinv(self.wexog)
            params = pinv_wexog.dot(self.wendog)
        elif method == 'qr':
            Q, R = np.linalg.qr(self.wexog)
            params = np.linalg.solve(R, np.dot(Q.T, self.wendog))
        else:
            params, _, _, _ = np.linalg.lstsq(self.wexog, self.wendog,
                                              rcond=-1)

        fitted_values = self.exog.dot(params)
        resid = self.endog - fitted_values
        wresid = self.wendog - self.wexog.dot(params)
        df_resid = self.wexog.shape[0] - self.wexog.shape[1]
        scale = np.dot(wresid, wresid) / df_resid

        return Bunch(params=params, fittedvalues=fitted_values, resid=resid,
                     model=self, scale=scale)



from distutils.version import LooseVersion

import numpy as np
import pandas as pd
import pytest
from numpy.testing import assert_allclose
from pandas.util.testing import assert_frame_equal, assert_series_equal

PD_GT_19 = LooseVersion(pd.__version__) > LooseVersion('0.19.2')

pytestmark = pytest.mark.skipif(PD_GT_19,
                                reason='Requires pandas <= 0.19.2')

from statsmodels.tsa.vector_ar.dynamic import _window_ols


@pytest.fixture(params=(0.0, 0.01))
def ols_data(request):
    nobs = 500
    rs = np.random.RandomState(12345)
    x = pd.DataFrame(rs.standard_normal((nobs, 3)),
                     columns=['x{0}'.format(i) for i in range(1, 4)])
    e = rs.standard_normal(500)
    y = pd.Series(x.values.sum(1) + e, name='y')

    if request.param > 0.0:
        locs = rs.random_sample(nobs)
        y.loc[locs < request.param] = np.nan

    return {'y': y, 'x': x}


def assert_ols_equal(res1, res2):
    if isinstance(res1.beta, pd.Series):
        assert_series_equal(res1.beta, res2.beta)
        assert_allclose(res1.r2, res2.r2)
        assert_allclose(res1.nobs, res2.nobs)
    else:
        assert_frame_equal(res1.beta, res2.beta)
        assert_series_equal(res1.nobs, res2.nobs)
        assert_series_equal(res1.r2, res2.r2)
    assert_series_equal(res1.resid, res2.resid)


@pytest.mark.filterwarnings('ignore::FutureWarning')
def test_window_ols_full(ols_data):
    y, x = ols_data['y'], ols_data['x']
    res1 = _window_ols(y, x, window_type='full_sample')
    res2 = _window_ols(y, x)
    res3 = pd.ols(y=y, x=x, window_type='full_sample')
    assert_ols_equal(res1, res2)
    assert_ols_equal(res1, res3)


@pytest.mark.filterwarnings('ignore::FutureWarning')
def test_window_ols_rolling(ols_data):
    y, x = ols_data['y'], ols_data['x']
    res1 = _window_ols(y, x, window_type='rolling', window=100)
    res2 = pd.ols(y=y, x=x, window_type='rolling', window=100)
    assert_ols_equal(res1, res2)


@pytest.mark.filterwarnings('ignore::FutureWarning')
def test_window_ols_expanding(ols_data):
    y, x = ols_data['y'], ols_data['x']
    res1 = _window_ols(y, x, window_type='expanding')
    res2 = pd.ols(y=y, x=x, window_type='expanding')
    assert_ols_equal(res1, res2)
